var Delivery = function()
{
	var delivery = {};
	var debug = 0;
	
	// Information about the currently logged in user
	var currentUser = null;
	// The last URL used for loading specimen 
	// We need it to be able to re-load the list after a download
	var loadUrl = null;
	// If the URL is loading the default (last 30 days) or has other search parameters
	var hasSearchParameters = false;
	
	// Months in the "select month" list
	var FIRST_YEAR = debug ? 2010 : 2016;
	var MONTHS = [
		'Januari', 'Februari', 'Mars', 'April', 'Maj', 'Juni',
		'Juli', 'Augusti', 'September', 'Oktober', 'November', 'December'
		];

	// Number of minutes before session timeout to display a warning
	var WARN_BEFORE_TIMEOUT = 5;
	
	// Page initialization
	delivery.initPage = function()
	{
		// The month selection list
		delivery.createMonthSelectionList();
		Events.addEventHandler('search-by-month', 'click', delivery.showMonthList);
		Events.addEventHandler(document.body, 'click', delivery.hideMonthList);
		Events.addEventHandler('month-list', 'click', delivery.searchByMonth);
		
		// The default search (last 30 days)
		Events.addEventHandler('search-last-30', 'click', delivery.searchLast30Days);
		
		// The manual search box
		Events.doOnEnter('search', delivery.searchReports);
		Events.addEventHandler('btnSearch', 'click', delivery.searchReports);
		
		// Logout button		
		Buttons.addClickHandler('btnLogout', delivery.logout);
		
		// Start timout check timer
		delivery.updateTimeout();

		// Perform the default search
		delivery.searchLast30Days();
	}
	
	/**
		Initialize the month selection list. 
	*/
	delivery.createMonthSelectionList = function()
	{
		var now = new Date();
		var year = now.getFullYear();
		var month = now.getMonth();

		var html = '';
		var numMonths = 0;
		while (year > FIRST_YEAR || year == FIRST_YEAR && month >= 0)
		{
			var monthClass = 'month';
			if (month < 0)
			{
				month = 11;
				year--;
				monthClass += ' new-year full-year';
			}
			if (numMonths == 0) monthClass += ' full-year';
			html += '<div class="'+monthClass+'" data-year="'+year+'" data-month="'+month+'">';
			if (month == 11 || numMonths == 0)
			{
				html += year + ' ';
			}
			html += MONTHS[month] + '</div>';
			month--;
			numMonths++;
		}
		
		Doc.element('month-list').innerHTML = html;
	}
	
	/**
		Display the month selection list. It is
		placed in relation to the current moust 
		position.
	*/
	delivery.showMonthList = function(event)
	{
		var monthList = Doc.element('month-list');
		var x = event.clientX;
		var y = event.clientY+2;

		monthList.style.left = (x)+'px';
		monthList.style.top = (y)+'px';
		event.stopPropagation();
		
		Doc.show(monthList);
	}
	
	/**
		Hide the month selection list.
	*/
	delivery.hideMonthList = function()
	{
		Doc.hide('month-list');
	}
	
	/**
		Search for samples from the given month that
		was clicked.
	 */
	delivery.searchByMonth = function(event)
	{
		var target = event.target;

		var month = Data.int(target, 'month');
		var year = Data.int(target, 'year');
		
		var title = 'Prov med ankomstdatum ' + MONTHS[month] + ' ' + year;
		var parameters = '&year='+year+'&month='+month;
		delivery.prepareForNewSearch(title, parameters);
	}
	
	/**
		Search for samples from the last 30 days (default search).
	*/
	delivery.searchLast30Days = function()
	{
		var title = 'Prov med ankomstdatum de senaste 30 dagarna';
		delivery.prepareForNewSearch(title, null);
	}

	/**
		Manual query for personal number, SCANB-ID, etc.
	 */
	delivery.searchReports = function()
	{
		var query = Doc.element('search').value;
		if (!query) 
		{
			Doc.element('search').focus();
			return;
		}
		
		var title = 'Prov som matchar \'' + Strings.encodeTags(query) + '\'';
		var parameters = '&query='+encodeURIComponent(query);
		delivery.prepareForNewSearch(title, parameters);
	}

	/**
		Prepare for a search by updating the title elements on
		the web page and generating the query url.
	*/
	delivery.prepareForNewSearch = function(title, urlParameters)
	{
		// Update titles and clear the search box
		if (title) delivery.setDocumentTitle(title);
		Doc.element('search').value = '';

		// Hide existing results...
		Doc.hide('new-reports-section');
		Doc.hide('work-in-progress-section');
		Doc.hide('archive-section');

		// Show a message for the user
		delivery.showMessage('Ett ögonblick, informationen hämtas...');
		
		loadUrl = '../Delivery.servlet?ID='+App.getSessionId();
		loadUrl += '&cmd=LoadYellowSpecimen';
		if (debug) loadUrl += '&debug=1';
		if (urlParameters)
		{
			loadUrl += urlParameters;
			hasSearchParameters = true;
		}
		else
		{
			hasSearchParameters = false;
		}
		delivery.loadSpecimenInformation();
	}

	/**
		Update titles on the web page. Typically used
		to reflect the current search.
	*/
	delivery.setDocumentTitle = function(title)
	{
		Doc.element('page-title').innerHTML = title;
		document.title = title;
	}
	
	/**
		Display a message for the user.
	*/
	delivery.showMessage = function(msg)
	{
		Doc.element('message-text').innerHTML = msg;
		Doc.show('message-section');
	}

	/**
		Set information about the current user.
	*/
	delivery.setUserInformation = function(userInfo)
	{
		currentUser = userInfo;
		Doc.element('current-username').innerHTML = Strings.encodeTags(currentUser.username);
		Doc.show('user-section');
	}
	
	/**
		Load specimen and report information.
	*/
	delivery.loadSpecimenInformation = function()
	{
		if (loadUrl == null)
		{
			loadUrl = '../Delivery.servlet?ID='+App.getSessionId();
			loadUrl += '&cmd=LoadYellowSpecimen';
		}
		Wizard.asyncJsonRequest(loadUrl, delivery.onSpecimenLoaded, 'GET', null, delivery.errorHandler);
	}

	/**
		Callback when specimen information has been successfully loaded.
	*/
	delivery.onSpecimenLoaded = function(response)
	{
		Doc.hide('message-section');
		
		// Update information about current user
		if (response.user) delivery.setUserInformation(response.user);
		
		// For debugging!
		//response.newReports = [];
		//response.workInProgress = [];
		//response.archived = [];
		
		// Generate a table with 3 sections: new reports, work in progres and archive.
		var html;
		var numReports = 0;
		numReports += response.newReports.length;
		html = delivery.createTBodySection(response.newReports, 'new-report', 'Nya rapporter', 'Det finns inga nya rapporter.');
		Doc.element('new-reports-section').innerHTML = html;
		Doc.showHide('new-reports-section', response.newReports.length > 0 || !hasSearchParameters);
		
		numReports += response.workInProgress.length;
		html = delivery.createTBodySection(response.workInProgress, 'work-in-progress', 'Arbete pågår');
		Doc.element('work-in-progress-section').innerHTML = html;
		Doc.showHide('work-in-progress-section', response.workInProgress.length > 0);
		
		numReports += response.archived.length;
		html = delivery.createTBodySection(response.archived, 'archived-report');
		Doc.element('archive-section').innerHTML = html;
		Doc.showHide('archive-section', response.archived.length > 0);
		
		if (numReports == 0) 
		{
			delivery.showMessage('Det finns inga rapporter.');
			Doc.hide('new-reports-section');
		}
		
		// Handle links to PDF documents
		var pdfLinks = document.getElementsByClassName('download-pdf');
		for (var pdfNo = 0; pdfNo < pdfLinks.length; pdfNo++)
		{
			Events.addEventHandler(pdfLinks[pdfNo], 'click', delivery.downloadPdf);
		}
	}

	
	/**
		Generate HTML for a section in the table.
		specimen = List of specimen information
		rowClass = Class attribute of each new <tr> tag
		header = Header text for the section (rotated; may be null)
		msgIfNoSpecimen = Message to display if the specimen list is emtpy
	*/
	delivery.createTBodySection = function(specimen, rowClass, header, msgIfNoSpecimen)
	{
		var html = '';
		if (specimen.length == 0 && msgIfNoSpecimen)
		{
			html += '<tr class="'+rowClass+'">';
			if (header)
			{
				html += '<td class="section-header small"></td>';
			}
			else
			{
				html += '<td></td>';
			}
			html += '<td colspan="8">' + msgIfNoSpecimen + '</td>';
			html += '</tr>';
		}
		
		for (var spNo = 0; spNo < specimen.length; spNo++)
		{
			var sp = specimen[spNo];
			var pat = sp.patient;
			var report = sp.pilotReport;
			
			html += '<tr class="'+rowClass+'">';
			if (header)
			{
				if (spNo == 0)
				{
					html += '<td rowspan="'+specimen.length+'" class="section-header">';
					html += '<div class="rotate">'+header+'</div>';
					html += '</td>';
				}
			}
			else
			{
				html += '<td></td>';
			}
			
			html += '<td>'+Strings.encodeTags(pat.PersonalNo)+'</td>';
			html += '<td>'+Strings.encodeTags(pat.AllFirstNames + ' ' + pat.FamilyName)+'</td>';
			html += '<td>'+Strings.encodeTags(sp.PAD)+'</td>';
			html += '<td>'+Strings.encodeTags(sp.name)+'</td>';
			html += '<td>'+Reggie.reformatDate(sp.ArrivalDate)+'</td>';
			
			if (report)
			{
				html += '<td>'+Reggie.reformatDate(report.lastUpdatedDate)+'</td>';
				if (report.downloadCount)
				{
					html += '<td>'+Reggie.reformatDate(report.downloadedDate)+' ('+report.downloadCount+')</td>';
				}
				else
				{
					html += '<td>Nej</td>';
				}
				html += '<td><img src="images/pdf-26.png" class="link download-pdf"';
				html += ' title="Ladda ner den här rapporten"';
				html += ' data-file-id="'+report.id+'" data-specimen-id="'+sp.id+'"></td>';
			}
			else if (sp.CompletedDate)
			{
				html += '<td class="completed-without-report" colspan="3">';
				html += 'Rapport saknas. Kontakta SCAN-B för information.</td>';
			}
			else
			{
				html += '<td colspan="3"></td>';
			}
			
			html += '</tr>';
		}
		
		return html;
	}

	/**
		Download a PDF document.
	*/
	delivery.downloadPdf = function(event)
	{
		var target = event.currentTarget;
		
		var url = '../Delivery.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadReport';
		url += '&fileId='+Data.int(target, 'file-id');
		url += '&specimenId='+Data.int(target, 'specimen-id');
		
		// The timeout will update the list with specimen
		// (since the report may move between sections)
		setTimeout(delivery.loadSpecimenInformation, 1500);
		
		window.open(url, 'DownloadPdf', 'width=600,height=400');
	}

	/**
		Logout.
	*/
	delivery.logout = function()
	{
		Doc.element('auto-logout').innerHTML = 'Loggar ut...';
		var url = '../Delivery.servlet?ID='+App.getSessionId();
		url += '&cmd=Logout';
		// Request for logging out, all responses will trigger the redirect
		Wizard.asyncJsonRequest(url, delivery.onLoggedOut, 'GET', null, delivery.onLoggedOut);
	}
	
	/**
		Redirect to the login page without history.
	*/
	delivery.onLoggedOut = function()
	{
		location.replace('index.jsp');
	}
	
	/**
		Update timeout information for the current session. If the
		timeout has been passed and automatick logout request is
		sent.
	*/
	delivery.updateTimeout = function()
	{
		if (currentUser) 
		{
			var timeout = currentUser.timeout;
			var now = new Date().getTime();
			
			if (timeout < now)
			{
				// Auto logout should happen now
				delivery.logout();
				return;
			}
			
			// Minutes to timeout rounded up
			var minutesToTimeout = Math.ceil((timeout - now) / 60000);
			// Seconds to timeout rounded up to nearest 15 seconds
			var secondsToTimeout = 15 * Math.ceil((timeout - now) / 15000);

			var autoLogout = '';
			if (minutesToTimeout > 1)
			{
				// There is still more than one minute 
				// Display a warning if it is less than the warning limit
				if (minutesToTimeout <= WARN_BEFORE_TIMEOUT)
				{
					autoLogout = 'Automatisk utloggning om <b>'+minutesToTimeout+' minuter</b>';
				}
			}
			else
			{
				autoLogout = 'Automatisk utloggning om <b>'+secondsToTimeout+' sekunder</b>';
			}
			Doc.element('auto-logout').innerHTML = autoLogout;
		}
		
		// Repeat every 5 seconds
		setTimeout(delivery.updateTimeout, 5000);
	}
	
	/**
		Error handler for AJAX requests
	*/
	delivery.errorHandler = function(response, error)
	{
		if (debug) 
		{
			App.debug(response);
			App.debug(error);
		}
		
		// Try to translate the error to Swedish
		var err = delivery.translateError(response ? response.message : error.toString());
	
		var msg = 'Det gick inte att hämta information om proven!';
		msg += '<div class="error-text" title="'+Strings.encodeTags(err.originalMessage)+'">';
		msg += Strings.encodeTags(err.message);
		msg += '</div>';

		delivery.showMessage(msg);
	}

	/**
		Translate some common error messages to Swedish. Returns an object with
		* originalMessage: The original error message
		* message: The translated error message (or the original if no translation was found)
		
		If the error message indicates that the session is no longer active the
		browser will be redirected to the login page.
	*/
	delivery.translateError = function(message)
	{
		var err = {};
		err.originalMessage = message;
		err.message = message;

		// The user has logged out!
		if (message.indexOf('Not logged in') >= 0 ||
			(message.indexOf('Item not found') >= 0 && message.indexOf('SessionControl') >= 0))
		{
			delivery.onLoggedOut();
		}
		
		if (message.indexOf('Permission denied') >= 0 || 
			(message.indexOf('Item not found') >= 0 && message.indexOf('AnnotationType') >= 0))
		{
			err.message = 'Behörighet saknas';
			
			if (message.indexOf('Biosource') >= 0)
			{
				err.message += ' för åtkomst till patient-information';
			}
			else if (message.indexOf('Sample') >= 0)
			{
				err.message += ' för åtkomst till prov-information';
			}
			else if (message.indexOf('Annotation') >= 0)
			{
				if (message.indexOf('PersonalNumber') >= 0 || 
					message.indexOf('AllFirstNames') >= 0 || 
					message.indexOf('FamilyName') >= 0)
				{
					err.message += ' för åtkomst till patient-information';
				}
				else
				{
					err.message += ' för åtkomst till prov-information';
				}
			}
		}

		return err;
	}

	return delivery;
}();

Doc.onLoad(Delivery.initPage);



