var LibReg = function()
{
	var libreg = {};
	var debug = 0;
	var isNeoPrep = 0;
	
	var libCommentsIsValid = false;
	
	// Page initialization
	libreg.initPage = function()
	{
		isNeoPrep = Data.int('page-data', 'is-neoprep');
		
		// Step 1
		Events.addEventHandler('bioplate', 'change', libreg.bioPlateOnChange);
		Events.addEventHandler('outcomeSuccess', 'click', libreg.outcomeOnChange);
		Events.addEventHandler('outcomeFailed', 'click', libreg.outcomeOnChange);
		Events.addEventHandler('step-1', 'wizard-validate', libreg.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', libreg.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', libreg.validateStep2);
		Events.addEventHandler('amplificationDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('cleanupDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('dilutionDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('neoPrepDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('libComments', 'blur', libreg.libCommentsOnChange);
		Wizard.initFileSelectionField('qubitcsv');
		Wizard.initFileSelectionField('welltable');
		Wizard.initFileSelectionField('calipergxd');
		Wizard.initFileSelectionField('caliperpdf');
		Events.addEventHandler('welltable', 'change', libreg.prevalidateWellTable);
		Events.addEventHandler('qubitcsv', 'change', libreg.prevalidateWellTable);
		Events.addEventHandler('libSize', 'change', Wizard.validateNumber);
		Events.addEventHandler('libSize', 'keypress', Events.numberOnly);
		
		if (debug)
		{
			Buttons.addClickHandler('btnFakeQubit', libreg.generateFakeFile);
			Buttons.addClickHandler('btnFakeWelltable', libreg.generateFakeFile);
		}

		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', libreg.submit);
		
		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLibraryPlatesForLibPrep';
		if (isNeoPrep)
		{
			url += '&plateType=NEOPREP';
			Doc.element('useCaliper').checked = false;
		}
		Wizard.showLoadingAnimation('Loading Library plates...');
		Wizard.asyncJsonRequest(url, libreg.initializeStep1);
	}


	libreg.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var bioplates = response.bioplates;

		var plates = frm.bioplate;
		if (bioplates.length > 0)
		{
			for (var i=0; i < bioplates.length; i++)
			{
				var bioplate = bioplates[i];
				var name = bioplate.name;
				if (isNeoPrep)
				{
					if (bioplate.AutoProcessing == 'PreNormalizeRNA')
					{
						name += ' (pre-normalized)';
					}
					else if (bioplate.DilutionDate)
					{
						name += ' (diluted ' + Reggie.reformatDate(bioplate.DilutionDate) + ')';
					}
				}
				var warning = bioplate.numFiles == 0 && !isNeoPrep ? ' ( ! )' : '';
				var option = new Option(name + warning, bioplate.id);
				option.bioplate = bioplate;
				plates.options[plates.length] = option;
			}
		}
		else
		{
			if (isNeoPrep)
			{
				Wizard.setFatalError('No NeoPrep bioplates available for processing.');
			}
			else
			{
				Wizard.setFatalError('No Library bioplates available for processing.');
			}
			return;
		}

		libreg.bioPlateOnChange();
		libreg.outcomeOnChange();
		frm.bioplate.focus();

		Doc.show('step-1');
		Doc.show('gonext');
	}
	
	libreg.bioPlateOnChange = function()
	{
		var frm = document.forms['reggie'];
		var selectedPlate = frm.bioplate[frm.bioplate.selectedIndex].bioplate;
		if (selectedPlate.numFiles == 0 && !isNeoPrep)
		{
			Wizard.setInputStatus('bioplate', 'warning', 'No quality control PDF has been registered for this plate');
		}
		else
		{
			Wizard.setInputStatus('bioplate', 'valid');
		}
	}

	libreg.outcomeOnChange = function()
	{
		var frm = document.forms['reggie'];
		var failed = Doc.element('outcomeFailed').checked
		Doc.element('useCaliper').disabled = failed;
		if (failed)
		{
			Doc.addClass(frm.libComments, 'required');
		}
		else
		{
			Doc.removeClass(frm.libComments, 'required');
		}
	}

	
	libreg.validateStep1 = function(event)
	{}
	
	libreg.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];	

		var failed = Doc.element('outcomeFailed').checked;
		var useCaliper = !failed && Doc.element('useCaliper').checked;
		Wizard.setCurrentStep(2);
		if (failed)
		{
			Wizard.showGoNextConfirmation(true, 'Check to verify registration of failure.');
			libreg.libCommentsOnChange();
			Doc.hide('caliperFilesSection');
			Doc.hide('qubitFilesSection');
			Doc.hide('noCaliperFilesSection');
		}
		else if (!useCaliper) 
		{
			Doc.hide('caliperFilesSection');
		}
		else
		{
			Doc.hide('noCaliperFilesSection');
		}
		
		Doc.show('gocancel');
		Doc.show('goregister');
		
		var libInfo = libreg.getStoredInfo(frm.bioplate.value);
		if (libInfo)
		{
			if (isNeoPrep)
			{
				if (libInfo.dilutionDate) 
				{
					frm.dilutionDate.value = libInfo.dilutionDate;
					frm.dilutionOperator.value = libInfo.dilutionOperator;
					frm.dilutionDate.focus();
				}
				if (libInfo.neoPrepDate) 
				{
					frm.neoPrepDate.value = libInfo.neoPrepDate;
					frm.neoPrepOperator.value = libInfo.neoPrepOperator;
					frm.neoPrepDate.focus();
				}
			}
			else
			{
				if (libInfo.amplificationDate) 
				{
					frm.amplificationDate.value = libInfo.amplificationDate;
					frm.amplificationOperator.value = libInfo.amplificationOperator;
					frm.amplificationDate.focus();
				}
				if (libInfo.cleanupDate) 
				{
					frm.cleanupDate.value = libInfo.cleanupDate;
					frm.cleanupOperator.value = libInfo.cleanupOperator;
					frm.cleanupDate.focus();
				}
			}
			if (!failed)
			{
				if (useCaliper)
				{
					if (libInfo.wellTable)
					{
						frm.welltable.value = libInfo.wellTable;
						Events.sendChangeEvent('welltable');
					}
					if (libInfo.caliperGxd)
					{
						frm.calipergxd.value = libInfo.caliperGxd;
						Events.sendChangeEvent('calipergxd');
					}
					if (libInfo.caliperPdf)
					{
						frm.caliperpdf.value = libInfo.caliperPdf;
						Events.sendChangeEvent('caliperpdf');
					}
				}
				else
				{
					Events.sendChangeEvent('libSize');	
				}
				if (libInfo.qubitCsv)
				{
					frm.qubitcsv.value = libInfo.qubitCsv;
					Events.sendChangeEvent('qubitcsv');
				}
			}
			if (libInfo.libComments) 
			{
				frm.libComments.value = libInfo.libComments;
				frm.libComments.focus();
			}
		}
		
		var plateName = frm.bioplate[frm.bioplate.selectedIndex].bioplate.name;
		Data.set('qubitcsv', 'name-filter', plateName + '%.csv');
		Data.set('welltable', 'name-filter', plateName + '%.csv');
		Data.set('calipergxd', 'name-filter', plateName + '%.gxd');
		Data.set('caliperpdf', 'name-filter', plateName + '%.pdf');
		
		if (isNeoPrep)
		{
			var url = '../LibPrep.servlet?ID='+App.getSessionId();
			url += '&cmd=GetLibraryInfoForPlate&bioplate='+frm.bioplate.value;
			url += '&loadDilutionInfo=1';
			Wizard.showLoadingAnimation('Loading NeoPrep plate information...');
			Wizard.asyncJsonRequest(url, libreg.libraryInfoLoaded);
		}
		else
		{
			frm.amplificationDate.focus();
		}

		// Load Library protocols
		Reggie.loadProtocols('LIBRARY_PROTOCOL', 'libProtocol');

		if (debug && !failed)
		{
			Wizard.showDebugTools();
		}
	}

	libreg.libraryInfoLoaded = function(response)
	{
		var frm = document.forms['reggie'];	
		
		var plate = response.plate;
		var libs = response.libraries;
		
		// Pre-process the return mRNA items
		var allPreNormalized = true;
		for (var i = 0; i < libs.length; i++)
		{
			allPreNormalized &= libs[i].rna.preNormalized;
		}
		
		if (plate.DilutionDate)
		{
			frm.dilutionDate.value = plate.DilutionDate;
			frm.dilutionOperator.value = Strings.encodeTags(plate.DilutionOperator);
		}
		if (allPreNormalized)
		{
			frm.dilutionDate.disabled = true;
			frm.dilutionOperator.disabled = true;
			frm.neoPrepDate.focus();
			Wizard.setInputStatus('dilutionDate', '', 'PreNormalized');
		}
		else
		{
			frm.dilutionDate.focus();
		}
	}

	libreg.prevalidateWellTable = function()
	{
		var frm = document.forms['reggie'];
		var useCaliper = frm.useCaliper.checked;

		var wellTable = frm.welltable.value;
		var qubitCsv = frm.qubitcsv.value;

		if (wellTable) Wizard.setInputStatus('welltable');
		if (qubitCsv) Wizard.setInputStatus('qubitcsv');
		Data.set('welltable', 'valid', 0);
		Data.set('qubitcsv', 'valid', 0);

		if (useCaliper && !wellTable || !qubitCsv) return;
		
		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd='+(useCaliper ? 'PreValidateCaliperWellTable' : 'PreValidateQubitCsv');
		url += '&bioplate='+frm.bioplate.value;
		if (useCaliper)
		{
			url += '&csv='+encodeURIComponent(wellTable);
		}
		url += '&qubit='+encodeURIComponent(qubitCsv);
		url += '&neoprep='+isNeoPrep;
		
		if (useCaliper) Wizard.setInputStatus('welltable', 'checking', 'Checking...');
		Wizard.setInputStatus('qubitcsv', 'checking', 'Checking...');
		Wizard.asyncJsonRequest(url, libreg.onWellTableValidated, 'POST');
	}

	libreg.onWellTableValidated = function(response)
	{
		var qubit = response.qubit;
		var welltable = response.welltable;
		
		Wizard.setInputStatus('welltable');
		Wizard.setInputStatus('qubitcsv');

		Wizard.showFinalMessage(response.messages);
		
		if (qubit.valid)
		{
			Wizard.setInputStatus('qubitcsv', qubit.warnings > 0 ? 'warning' : 'valid');
			Data.set('qubitcsv', 'valid', 1);
		}
		else
		{
			Wizard.setInputStatus('qubitcsv', 'invalid');
		}
		
		if (welltable)
		{
			if (welltable.valid)
			{
				Wizard.setInputStatus('welltable', welltable.warnings > 0 ? 'warning' : 'valid');
				Data.set('welltable', 'valid', 1);
			}
			else
			{
				Wizard.setInputStatus('welltable', 'invalid');
			}
		}
	}

	var fileField;
	libreg.generateFakeFile = function(event)
	{
		var target = event.currentTarget;
		
		if (location.protocol == 'https:')
		{
			Forms.showNotification(target, "For security reasons, this function can't be used on HTTPS servers.");
			return;
		}
		
		var frm = document.forms['reggie'];
		fileField = Data.get(target, 'file-field');
		var path = Data.get(fileField, 'directory');

		var url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd='+Data.get(target, 'cmd');
		url += '&bioplate='+frm.bioplate.value;
		url += '&path='+encodeURIComponent(path);
		
		Wizard.setInputStatus(fileField, 'checking', 'Generating fake file...');
		Wizard.asyncJsonRequest(url, libreg.fakeFileGenerated, 'POST');
	}
	
	libreg.fakeFileGenerated = function(response)
	{
		var frm = document.forms['reggie'];
		var file = response.file;
		frm[fileField].value = file.path;
		Events.sendChangeEvent(fileField);
	}

	libreg.libCommentsOnChange = function()
	{
		var failed = Doc.element('outcomeFailed').checked;
		if (!failed) return;

		var frm = document.forms['reggie'];
		libCommentsIsValid = false;
		Wizard.setInputStatus('libComments');
		
		var comments = frm.libComments.value;
		if (comments == '')
		{
			Wizard.setInputStatus('libComments', 'invalid', 'Missing');
			return;
		}

		Wizard.setInputStatus('libComments', 'valid');
		libCommentsIsValid = true;
	}

	libreg.validateStep2 = function(event)
	{
		var valid = true;
		
		if (isNeoPrep)
		{
			valid &= Wizard.isValid('dilutionDate');
			valid &= Wizard.isValid('neoPrepDate');
		}
		else
		{
			valid &= Wizard.isValid('amplificationDate');
			valid &= Wizard.isValid('cleanupDate');
		}
		
		var failed = Doc.element('outcomeFailed').checked;
		var useCaliper = Doc.element('useCaliper').checked;
		if (failed)
		{
			valid &= libCommentsIsValid;
		}
		else
		{
			valid &= Wizard.isValid('qubitcsv');
			if (useCaliper)
			{
				valid &= Wizard.isValid('welltable');
			}
			else
			{
				valid &= Wizard.isValid('libSize');
			}
		}
		
		if (!valid) event.preventDefault();
	}

	
	libreg.submit = function()
	{
		var frm = document.forms['reggie'];

		var submitInfo = {};
		submitInfo.bioplate = parseInt(frm.bioplate.value, 10);

		var failed = Doc.element('outcomeFailed').checked;
		var useCaliper = Doc.element('useCaliper').checked;
		submitInfo.failed = failed;
			
		if (!failed)
		{
			libreg.saveToLocalStorage(frm.bioplate.value);
			if (useCaliper)
			{
				submitInfo.wellTableCsv = frm.welltable.value;
				submitInfo.caliperGxd = frm.calipergxd.value;
				submitInfo.caliperPdf = frm.caliperpdf.value;
			}
			else
			{
				submitInfo.libSize = parseFloat(frm.libSize.value);
			}
			submitInfo.qubitCsv = frm.qubitcsv.value;
		}
		if (isNeoPrep)
		{
			submitInfo.dilutionDate = frm.dilutionDate.value;
			submitInfo.dilutionOperator = frm.dilutionOperator.value;
			submitInfo.neoPrepDate = frm.neoPrepDate.value;
			submitInfo.neoPrepOperator = frm.neoPrepOperator.value;
		}
		else
		{
			submitInfo.amplificationDate = frm.amplificationDate.value;
			submitInfo.amplificationOperator = frm.amplificationOperator.value;
			submitInfo.cleanupDate = frm.cleanupDate.value;
			submitInfo.cleanupOperator = frm.cleanupOperator.value;
		}
		submitInfo.libProtocol = parseInt(frm.libProtocol.value, 10);
		submitInfo.libComments = frm.libComments.value;
		
		var	url = '../LibPrep.servlet?ID='+App.getSessionId();
		url += '&cmd=ImportLibPrepResults';
		url += '&neoprep='+isNeoPrep;
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, libreg.submissionResults, 'POST', JSON.stringify(submitInfo));
	}

	libreg.submissionResults = function(response)
	{
		var frm = document.forms['reggie'];
		libreg.removeFromLocalStorage(frm.bioplate.value);
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	/**
		Save as much information as possible about the library registration.
		The information is stored in the localStorage()
		under the key 'reggie.libplate.<id>.info' where
		<id> is the ID of the library plate.
	*/
	libreg.saveToLocalStorage = function(libPlateId)
	{
		if (!window.localStorage) return;

		var frm = document.forms['reggie'];
		var libInfo = {};
		libInfo.id = libPlateId;
		if (isNeoPrep)
		{
			libInfo.dilutionDate = frm.dilutionDate.value;
			libInfo.dilutionOperator = frm.dilutionOperator.value;
			libInfo.neoPrepDate = frm.neoPrepDate.value;
			libInfo.neoPrepOperator = frm.neoPrepOperator.value;
		}
		else
		{
			libInfo.amplificationDate = frm.amplificationDate.value;
			libInfo.amplificationOperator = frm.amplificationOperator.value;
			libInfo.cleanupDate = frm.cleanupDate.value;
			libInfo.cleanupOperator = frm.cleanupOperator.value;
		}
		libInfo.wellTable = frm.welltable.value;
		libInfo.caliperGxd = frm.calipergxd.value;
		libInfo.caliperPdf = frm.caliperpdf.value;
		libInfo.qubitCsv = frm.qubitcsv.value;
		libInfo.libProtocol = frm.libProtocol.value;
		libInfo.libComments = frm.libComments.value;
		
		window.localStorage.setItem('reggie.libplate.' + libPlateId + '.info', JSON.stringify(libInfo));
		if (debug)
		{
			App.debug(libPlateId + ': ' + window.localStorage.getItem('reggie.libplate.' + libPlateId + '.info'));
		}
	}
	

	libreg.removeFromLocalStorage = function(libPlateId)
	{
		if (!window.localStorage) return;
		if (debug) App.debug('removeFromLocalStorage:' + libPlateId);
		window.localStorage.removeItem('reggie.libplate.' + libPlateId + '.info');
	}

	libreg.getStoredInfo = function(libPlateId)
	{
		if (!window.localStorage) return null;
		var stored = null;
		try
		{
			var info = window.localStorage.getItem('reggie.libplate.' + libPlateId + '.info');
			if (debug) App.debug(libPlateId + ': ' + info);
			stored = JSON.parse(info);
		}
		catch (e)
		{
			App.debug('Could not load stored information for lib plate ' + libPlateId + ': ' + e);
		}
		return stored;
	}

	
	return libreg;
}();

Doc.onLoad(LibReg.initPage);

